__title__ = 'fobi.contrib.plugins.form_elements.fields.file.defaults'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('FILES_UPLOAD_DIR',)

import os

FILES_UPLOAD_DIR = os.path.join('fobi_plugins', 'file')
